
/* This program reproduces the tables in the paper "Understanding the risk-return relation: The aggregate wealth proxy actually matters" by Scott Cederburg and Michael S. O'Doherty.

Description of SAS datasets:
	[Required for Tabs. 1-5]
	 (1) main.sas7bdat - Monthly data on excess returns (e.g., ce) and aggregate wealth portfolio weights (e.g., wce) for the following asset classes: corporate equity (ce), 
		 noncorporate equity (nce), deposits (mm), treasury debt (td), agency debt (ad), municipal debt (md), corporate debt (cd), pensions (pen), and real estate (re); monthly 
		 excess returns for the stock market portfolio (mkt1) and the aggregate wealth portfolio (mkt2).
	
	[Required for Tab. 3]
	 (2) bekk_diagonal_stocks.sas7bdat - Monthly data on the conditional covariance between stock market returns and aggregate wealth returns (cov).

	[Required for Tab. 4]
	 (3) bekk_pairwise_all.sas7bdat - Monthly data on conditional covariances between aggregate wealth returns and returns for the following eight asset classes: corporate equity 
		 (covce), noncorporate equity (covnce), treasury debt (covtd), agency debt (covad), municipal debt (covmd), corporate debt (covcd), pensions (covpen), and real estate 
		 (covre).

	[Required for Tab. 5]
	 (4) portfolios.sas7bdat - Monthly data on excess returns for decile portfolios sorted on size (s1-s10), book-to-market equity (bm1-bm10), and momentum (m1-m10).
	 (5) bekk_anomalies_size_awpproxy.sas7bdat - Monthly data on the conditional covariances between aggregate wealth returns and returns for size decile portfolios (cov1-cov10);
	 (6) bekk_anomalies_size_stocksproxy.sas7bdat - Monthly data on the conditional covariances between stock market returns and returns for size decile portfolios (cov1-cov10);
	 (7) bekk_anomalies_bm_awpproxy.sas7bdat - Monthly data on the conditional covariances between aggregate wealth returns and returns for book-to-market decile portfolios (cov1-cov10);
	 (8) bekk_anomalies_bm_stocksproxy.sas7bdat - Monthly data on the conditional covariances between stock market returns and returns for book-to-market decile portfolios (cov1-cov10);
	 (9) bekk_anomalies_mom_awpproxy.sas7bdat - Monthly data on the conditional covariances between aggregate wealth returns and returns for momentum decile portfolios (cov1-cov10);
    (10) bekk_anomalies_mom_stocksproxy.sas7bdat - Monthly data on the conditional covariances between stock market returns and returns for momentum decile portfolios (cov1-cov10); */

* To run the program, define the SAS library associated with the physical location of the files listed above, e.g.,;
libname co 'C:\Folder_Name';


* Tab. 1: Summary statistics for aggregate wealth portfolio and component asset classes
	(a) Market portfolio weights 
	(b) Aggregate wealth returns and component returns 
	(c) Return correlations;

data co.temp1; set co.main; if mo in(1,4,7,10); run;
proc means noprint data = co.temp1; var wce wnce wmm wtd wad wmd wcd wpen wre; output out = co.temp2 mean(wce wnce wmm wtd wad wmd wcd wpen wre) = wce wnce wmm wtd wad wmd wcd wpen wre
																								 min(wce wnce wmm wtd wad wmd wcd wpen wre) = mnce mnnce mnmm mntd mnad mnmd mncd mnpen mnre
																								 max(wce wnce wmm wtd wad wmd wcd wpen wre) = mxce mxnce mxmm mxtd mxad mxmd mxcd mxpen mxre; quit;
data co.temp2; set co.temp2; drop _type_ _freq_; run;
proc iml; reset log noprint; 
	use co.temp2; read all into A;
	B = j(3,9,0);
	B[1,1:9] = A[1,1:9]; B[2,1:9] = A[1,10:18]; B[3,1:9] = A[1,19:27];
	B = 100*B;
	create co.temp3 from B; append from B; quit;
data co.temp3; set co.temp3 (rename = (COL1 = wce COL2 = wnce COL3 = wmm COL4 = wtd COL5 = wad COL6 = wmd COL7 = wcd COL8 = wpen COL9 = wre)); run;
data co.temp3; set co.temp3;
	if _n_ = 1 then stat = 'Mean   ';
	if _n_ = 2 then stat = 'Minimun';
	if _n_ = 3 then stat = 'Maximum'; run;
proc print noobs data = co.temp3; format wce wnce wmm wtd wad wmd wcd wpen wre 8.2; var stat wce wnce wmm wtd wad wmd wcd wpen wre; quit;
proc datasets noprint lib = co; delete temp1-temp3; quit;

data co.temp1; set co.main; run;
proc means noprint data = co.temp1; var mkt2 ce nce mm td ad md cd pen re; output out = co.temp2 mean(mkt2 ce nce mm td ad md cd pen re) = mkt2 ce nce mm td ad md cd pen re
																								 std(mkt2 ce nce mm td ad md cd pen re) = smkt2 sce snce smm std sad smd scd spen sre
																								 var(mkt2 ce nce mm td ad md cd pen re) = vmkt2 vce vnce vmm vtd vad vmd vcd vpen vre
																								 min(mkt2 ce nce mm td ad md cd pen re) = mnmkt2 mnce mnnce mnmm mntd mnad mnmd mncd mnpen mnre
																								 max(mkt2 ce nce mm td ad md cd pen re) = mxmkt2 mxce mxnce mxmm mxtd mxad mxmd mxcd mxpen mxre; quit;
data co.temp2; set co.temp2; drop _type_ _freq_; run;
proc iml; reset log noprint; 
	use co.temp2; read all into A;
	B = j(6,10,0);
	B[1,1:10] = A[1,1:10]; B[2,1:10] = A[1,11:20]; B[3,1:10] = A[1,21:30]; B[4,1:10] = A[1,31:40]; B[5,1:10] = A[1,41:50];
	do i = 1 to 10; B[6,i] = B[1,i]/B[2,i]; end;
	create co.temp3 from B; append from B; quit;
data co.temp3; set co.temp3 (rename = (COL1 = mkt2 COL2 = ce COL3 = nce COL4 = mm COL5 = td COL6 = ad COL7 = md COL8 = cd COL9 = pen COL10 = re)); run;
data co.temp3; set co.temp3;
	if _n_ = 1 then stat = 'Mean    ';
	if _n_ = 2 then stat = 'StDev   ';
	if _n_ = 3 then stat = 'Variance';
	if _n_ = 4 then stat = 'Minimun ';
	if _n_ = 5 then stat = 'Maximum '; 
	if _n_ = 6 then stat = 'Sharpe  '; run;
data co.temp1; set co.temp1; mkt21 = lag(mkt2); mm1 = lag(mm); td1 = lag(td); ad1 = lag(ad); md1 = lag(md); cd1 = lag(cd); ce1 = lag(ce); nce1 = lag(nce); pen1 = lag(pen); re1 = lag(re); run;
proc corr noprint outp = co.temp2 data = co.temp1; var mkt2 ce nce mm td ad md cd pen re mkt21 ce1 nce1 mm1 td1 ad1 md1 cd1 pen1 re1; quit;
data co.temp2; set co.temp2; drop _type_ _name_; run;
proc iml; reset log noprint; 
	use co.temp2; read all into A;
	B = j(1,10,0);
	B[1,1:10] = (A[14,1]||A[15,2]||A[16,3]||A[17,4]||A[18,5]||A[19,6]||A[20,7]||A[21,8]||A[22,9]||A[23,10]);
	create co.temp4 from B; append from B; quit;
data co.temp4; set co.temp4 (rename = (COL1 = mkt2 COL2 = ce COL3 = nce COL4 = mm COL5 = td COL6 = ad COL7 = md COL8 = cd COL9 = pen COL10 = re)); run;
data co.temp4; set co.temp4;
	if _n_ = 1 then stat = 'Autocorrelation'; run;
data co.temp5; set co.temp3 co.temp4; run;
proc print noobs data = co.temp5; format mkt2 ce nce mm td ad md cd pen re 8.2; var stat mkt2 ce nce mm td ad md cd pen re; quit;
proc corr noprint outp = co.temp2 data = co.temp1; var mkt2 ce nce mm td ad md cd pen re; quit;
data co.temp2; set co.temp2; if _type_ = 'CORR'; run;
proc print noobs data = co.temp2; format mkt2 ce nce mm td ad md cd pen re 8.2; var _type_ _name_ mkt2 ce nce mm td ad md cd pen re; quit;
proc datasets noprint lib = co; delete temp1-temp5; quit;


* Tab. 2: The risk-return tradeoff for the stock market portfolio and aggregate wealth portfolio 
	(a) Stock market portfolio, 2-stage GARCH
	(b) Aggregate wealth portfolio, 2-stage GARCH;

data co.temp1; set co.main; mkt1 = 0.01*mkt1; mkt2 = 0.01*mkt2; run; 
proc reg noprint data = co.temp1; model mkt1 =; output out = co.temp2 r = market; quit;
proc autoreg data = co.temp2; model market =/ noint garch=(p=1,q=1) maxit = 1000; output out = co.temp2 cev = v r = res; run;
data co.temp2; set co.temp2; lmkt = lag(mkt1); run;
proc iml;
	use co.temp2; read all var{mkt1 v mkt2} into Z; 
	max = nrow(Z);
	iter = 100000;
	F = j(3,5,{.});
	A = j(iter,4,0);
	B = j(iter,3,0);
	X = j(max,3,0);

	y1 = Z[,1]; x1 = j(max,1,1)||Z[,3]; beta = inv(x1`*x1)*x1`*y1; beta = beta[2]; rho = corr(Z[,1]||Z[,3]); rho = rho[2,1];
	y1 = Z[,1]; x1 = j(max,1,1)||Z[,2]; b1 = inv(x1`*x1)*x1`*y1; e = y1-x1*b1;
	F[1,1] = 100*b1[1]; F[1,2] = b1[2]; F[1,4] = 100*(1 - e`*e/(y1`*(I(nrow(y1))-(1/nrow(y1))*j(nrow(y1),1,1)*j(1,nrow(y1),1))*y1));
	F[1,5] = beta*b1[2]*rho##-2;

	G = beta||rho;
	names1 = {beta rho}; 
	print G[label = "Bias parameters" format = 6.3 colname=names1];

	do i = 1 to iter; u = j(max,1); call randseed(47,0); call randgen(u, "Uniform"); k = ceil(max*u);
		do j = 1 to max; X[j,] = Z[k[j,],]; end; 
		y1 = X[,1]; x1 = j(max,1,1)||X[,2]; x2 = j(max,1,1)||X[,3];
		b1 = inv(x1`*x1)*x1`*y1; A[i,1] = b1[2]; if A[i,1] < 0 then A[i,2] = 1; B[i,1:2] = b1`;
		beta = inv(x2`*x2)*x2`*y1; beta = beta[2]; rho = corr(X[,1]||X[,3]); rho = rho[2,1]; A[i,3] = beta*b1[2]*rho##-2; if A[i,3] < 0 then A[i,4] = 1; B[i,3] = A[i,3]; 
	end;
	se = std(B);
	F[2,1] = 100*se[1]; F[2,2] = se[2]; F[2,5] = se[3];
	F[3,2] = A[:,2]; F[3,5] = A[:,4]; 
	
	names0 = {coeff se pval}; names1 = {mu gams lam rsq gamaw}; 
	print F[label = "Two-step approach, Stocks" format = 6.3 rowname=names0 colname=names1];
quit;
data co.temp2; set co.temp2; if _n_ = 1 then delete; run;
proc iml;
	use co.temp2; read all var{mkt1 v lmkt mkt2} into Z; 
	max = nrow(Z);
	iter = 100000;
	F = j(3,5,{.});
	A = j(iter,4,0);
	B = j(iter,4,0);
	X = j(max,4,0);

	y1 = Z[,1]; x1 = j(max,1,1)||Z[,4]; beta = inv(x1`*x1)*x1`*y1; beta = beta[2]; rho = corr(Z[,1]||Z[,4]); rho = rho[2,1];
	y1 = Z[,1]; x1 = j(max,1,1)||Z[,2:3]; b1 = inv(x1`*x1)*x1`*y1; e = y1-x1*b1;
	F[1,1] = 100*b1[1]; F[1,2] = b1[2]; F[1,3] = b1[3]; F[1,4] = 100*(1 - e`*e/(y1`*(I(nrow(y1))-(1/nrow(y1))*j(nrow(y1),1,1)*j(1,nrow(y1),1))*y1));
	F[1,5] = beta*b1[2]*rho##-2;

	do i = 1 to iter; u = j(max,1); call randseed(47,0); call randgen(u, "Uniform"); k = ceil(max*u);
		do j = 1 to max; X[j,] = Z[k[j,],]; end; 
		y1 = X[,1]; x1 = j(max,1,1)||X[,2:3]; x2 = j(max,1,1)||X[,4];
		b1 = inv(x1`*x1)*x1`*y1; A[i,1] = b1[2]; if A[i,1] < 0 then A[i,2] = 1; B[i,1:3] = b1`;
		beta = inv(x2`*x2)*x2`*y1; beta = beta[2]; rho = corr(X[,1]||X[,4]); rho = rho[2,1]; A[i,3] = beta*b1[2]*rho##-2; if A[i,3] < 0 then A[i,4] = 1; B[i,4] = A[i,3]; 
	end;
	se = std(B);
	F[2,1] = 100*se[1]; F[2,2] = se[2]; F[2,3] = se[3]; F[2,5] = se[4];
	F[3,2] = A[:,2]; F[3,5] = A[:,4]; 
	
	names0 = {coeff se pval}; names1 = {mu gams lam rsq gamaw}; 
	print F[label = "Two-step approach, Stocks" format = 6.3 rowname=names0 colname=names1];
quit;
proc datasets noprint lib = co; delete temp1-temp4; quit;

data co.temp1; set co.main; mkt1 = 0.01*mkt1; mkt2 = 0.01*mkt2; run; 
proc reg noprint data = co.temp1; model mkt1 =; output out = co.temp2 r = market; quit;
proc autoreg noprint data = co.temp2; model market =/ noint garch=(p=1,q=1) maxit = 1000; output out = co.temp3 cev = v r = res; run;
data co.temp3; set co.temp3; lmkt = lag(mkt1); run;
proc reg noprint data = co.temp1; model mkt2 =; output out = co.temp2 r = market; quit;
proc autoreg data = co.temp2; model market =/ noint garch=(p=1,q=1) maxit = 1000; output out = co.temp4 cev = v r = res; run;
data co.temp4; set co.temp4; lmkt = lag(mkt2); run;
proc iml;
	use co.temp3; read all var{mkt1 v} into Z1; use co.temp4; read all var{mkt2 v} into Z2; Z = Z1||Z2;
	max = nrow(Z);
	iter = 100000;
	F = j(3,4,{.});
	A = j(iter,6,0);
	B = j(iter,2,0);
	X = j(max,4,0);

	y1 = Z[,3]; x1 = j(max,1,1)||Z[,4]; b1 = inv(x1`*x1)*x1`*y1; e = y1-x1*b1;
	F[1,1] = 100*b1[1]; F[1,2] = b1[2]; F[1,4] = 100*(1 - e`*e/(y1`*(I(nrow(y1))-(1/nrow(y1))*j(nrow(y1),1,1)*j(1,nrow(y1),1))*y1));

	do i = 1 to iter; u = j(max,1); call randseed(47,0); call randgen(u, "Uniform"); k = ceil(max*u);
		do j = 1 to max; X[j,] = Z[k[j,],]; end; 
		y1 = X[,1]; x1 = j(max,1,1)||X[,2]; y2 = X[,3]; x2 = j(max,1,1)||X[,4]; 
		b1 = inv(x1`*x1)*x1`*y1; A[i,1] = b1[2,]; if A[i,1] < 0 then A[i,2] = 1; 
		b2 = inv(x2`*x2)*x2`*y2; A[i,3] = b2[2,]; if A[i,3] < 0 then A[i,4] = 1;
		A[i,5] = b2[2,]-b1[2,]; if A[i,5] < 0 then A[i,6] = 1;
		B[i,1:2] = b2`;
	end;
	se = std(B);
	F[2,1] = 100*se[1]; F[2,2] = se[2];
	F[3,2] = A[:,4]; 
	
	names0 = {coeff se pval}; names1 = {mu gamaw lam rsq}; 
	print F[label = "Two-step approach, AWP" format = 6.3 rowname=names0 colname=names1];

	G = A[:,6];
	names1 = {pdelta}; 
	print G[label = "Difference" format = 6.3 colname=names1];
quit;
data co.temp3; set co.temp3; if _n_ = 1 then delete; run;
data co.temp4; set co.temp4; if _n_ = 1 then delete; run;
proc iml;
	use co.temp3; read all var{mkt1 v lmkt} into Z1; use co.temp4; read all var{mkt2 v lmkt} into Z2; Z = Z1||Z2;
	max = nrow(Z);
	iter = 100000;
	F = j(3,4,{.});
	A = j(iter,6,0);
	B = j(iter,3,0);
	X = j(max,6,0);

	y1 = Z[,4]; x1 = j(max,1,1)||Z[,5:6]; b1 = inv(x1`*x1)*x1`*y1; e = y1-x1*b1;
	F[1,1] = 100*b1[1]; F[1,2] = b1[2]; F[1,3] = b1[3]; F[1,4] = 100*(1 - e`*e/(y1`*(I(nrow(y1))-(1/nrow(y1))*j(nrow(y1),1,1)*j(1,nrow(y1),1))*y1));

	do i = 1 to iter; u = j(max,1); call randseed(47,0); call randgen(u, "Uniform"); k = ceil(max*u);
		do j = 1 to max; X[j,] = Z[k[j,],]; end; 
		y1 = X[,1]; x1 = j(max,1,1)||X[,2:3]; y2 = X[,4]; x2 = j(max,1,1)||X[,5:6]; 
		b1 = inv(x1`*x1)*x1`*y1; A[i,1] = b1[2,]; if A[i,1] < 0 then A[i,2] = 1; 
		b2 = inv(x2`*x2)*x2`*y2; A[i,3] = b2[2,]; if A[i,3] < 0 then A[i,4] = 1;
		A[i,5] = b2[2,]-b1[2,]; if A[i,5] < 0 then A[i,6] = 1;
		B[i,1:3] = b2`;
	end;
	se = std(B);
	F[2,1] = 100*se[1]; F[2,2] = se[2]; F[2,3] = se[3];
	F[3,2] = A[:,4]; 
	
	names0 = {coeff se pval}; names1 = {mu gamaw lam rsq}; 
	print F[label = "Two-step approach, AWP" format = 6.3 rowname=names0 colname=names1];

	G = A[:,6];
	names1 = {pdelta}; 
	print G[label = "Difference" format = 6.3 colname=names1];
quit;
proc datasets noprint lib = co; delete temp1-temp4; quit;


* Tab. 3: The risk-return tradeoff for the stock market portfolio: Covariance approach;

data co.temp1; set co.main; mkt1 = 0.01*mkt1; run; 
data co.temp2; set co.bekk_diagonal_stocks; run;
data co.temp3; merge co.temp1 co.temp2; by yr mo; run;
data co.temp3; set co.temp3; lmkt = lag(mkt1); run;
proc reg noprint data = co.temp1; model mkt1 =; output out = co.temp2 r = market; quit;
proc autoreg noprint data = co.temp2; model market =/ noint garch=(p=1,q=1) maxit = 1000; output out = co.temp2 cev = v r = res; run;
data co.temp2; set co.temp2; keep v; run;
data co.temp3; merge co.temp3 co.temp2; run;
proc iml;
	use co.temp3; read all var{mkt1 v} into Z1; use co.temp3; read all var{mkt1 cov} into Z2; Z = Z1||Z2;
	max = nrow(Z);
	iter = 100000;
	F = j(3,4,{.});
	A = j(iter,6,0);
	B = j(iter,2,0);
	X = j(max,4,0);

	y1 = Z[,3]; x1 = j(max,1,1)||Z[,4]; b1 = inv(x1`*x1)*x1`*y1; e = y1-x1*b1;
	F[1,1] = 100*b1[1]; F[1,2] = b1[2]; F[1,4] = 100*(1 - e`*e/(y1`*(I(nrow(y1))-(1/nrow(y1))*j(nrow(y1),1,1)*j(1,nrow(y1),1))*y1));

	do i = 1 to iter; u = j(max,1); call randseed(47,0); call randgen(u, "Uniform"); k = ceil(max*u);
		do j = 1 to max; X[j,] = Z[k[j,],]; end; 
		y1 = X[,1]; x1 = j(max,1,1)||X[,2]; y2 = X[,3]; x2 = j(max,1,1)||X[,4]; 
		b1 = inv(x1`*x1)*x1`*y1; A[i,1] = b1[2,]; if A[i,1] < 0 then A[i,2] = 1; 
		b2 = inv(x2`*x2)*x2`*y2; A[i,3] = b2[2,]; if A[i,3] < 0 then A[i,4] = 1;
		A[i,5] = b2[2,]-b1[2,]; if A[i,5] < 0 then A[i,6] = 1;
		B[i,1:2] = b2`;
	end;
	se = std(B);
	F[2,1] = 100*se[1]; F[2,2] = se[2];
	F[3,2] = A[:,4]; 
	
	names0 = {coeff se pval}; names1 = {mu gamaw lam rsq}; 
	print F[label = "Two-step approach, Covariance" format = 6.3 rowname=names0 colname=names1];

	G = A[:,6];
	names1 = {pdelta}; 
	print G[label = "Difference" format = 6.3 colname=names1];
quit;
data co.temp3; set co.temp3; if _n_ = 1 then delete; run;
proc iml;
	use co.temp3; read all var{mkt1 v lmkt} into Z1; use co.temp3; read all var{mkt1 cov lmkt} into Z2; Z = Z1||Z2;
	max = nrow(Z);
	iter = 100000;
	F = j(3,4,{.});
	A = j(iter,6,0);
	B = j(iter,3,0);
	X = j(max,6,0);

	y1 = Z[,4]; x1 = j(max,1,1)||Z[,5:6]; b1 = inv(x1`*x1)*x1`*y1; e = y1-x1*b1;
	F[1,1] = 100*b1[1]; F[1,2] = b1[2]; F[1,3] = b1[3]; F[1,4] = 100*(1 - e`*e/(y1`*(I(nrow(y1))-(1/nrow(y1))*j(nrow(y1),1,1)*j(1,nrow(y1),1))*y1));

	do i = 1 to iter; u = j(max,1); call randseed(47,0); call randgen(u, "Uniform"); k = ceil(max*u);
		do j = 1 to max; X[j,] = Z[k[j,],]; end; 
		y1 = X[,1]; x1 = j(max,1,1)||X[,2:3]; y2 = X[,4]; x2 = j(max,1,1)||X[,5:6]; 
		b1 = inv(x1`*x1)*x1`*y1; A[i,1] = b1[2,]; if A[i,1] < 0 then A[i,2] = 1; 
		b2 = inv(x2`*x2)*x2`*y2; A[i,3] = b2[2,]; if A[i,3] < 0 then A[i,4] = 1;
		A[i,5] = b2[2,]-b1[2,]; if A[i,5] < 0 then A[i,6] = 1;
		B[i,1:3] = b2`;
	end;
	se = std(B);
	F[2,1] = 100*se[1]; F[2,2] = se[2]; F[2,3] = se[3];
	F[3,2] = A[:,4]; 
	
	names0 = {coeff se pval}; names1 = {mu gamaw lam rsq}; 
	print F[label = "Two-step approach, Covariance" format = 6.3 rowname=names0 colname=names1];

	G = A[:,6];
	names1 = {pdelta}; 
	print G[label = "Difference" format = 6.3 colname=names1];
quit;
proc datasets noprint lib = co; delete temp1-temp3; quit;


* Tab. 04: The risk-return tradeoff across asset classes: Covariance approach;

data co.temp1; set co.main; mkt2 = 0.01*mkt2; ce = 0.01*ce; nce = 0.01*nce; td = 0.01*td; cd = 0.01*cd; pen = 0.01*pen; re = 0.01*re; ad = 0.01*ad; md = 0.01*md; run; 
proc reg noprint data = co.temp1; model mkt2 =; output out = co.temp2 r = market2; quit;
proc autoreg noprint data = co.temp2; model market2 =/ noint garch=(p=1,q=1) maxit = 1000; output out = co.temp1 cev = vaw r = res2; run;
data co.temp2; set co.bekk_pairwise_all; run;
data co.temp3; merge co.temp1 co.temp2; by yr mo; run;
data co.temp3; set co.temp3; lmkt = lag(mkt2); lce = lag(ce); lnce = lag(nce); ltd = lag(td); lcd = lag(cd); lpen = lag(pen); lre = lag(re); lad = lag(ad); lmd = lag(md); run;
data co.temp4; set co.temp3; if ad = . then delete; run;
data co.temp5; set co.temp3; if md = . then delete; run;
proc iml;
	use co.temp3; read all var{mkt2 ce nce td cd pen re  vaw covce covnce covtd covcd covpen covre} into Za;
	use co.temp4; read all var{ad covad} into Zb;
	use co.temp5; read all var{md covmd} into Zc;

	F = j(3,11,{.});
	iterout = 10000;
	iterin = 10000;

	A = j(iterout,20,0);
	B = j(iterin,10,0); Btest = j(iterin,10,0);

	na = nrow(Za); nb = nrow(Zb); nc = nrow(Zc);
	ba = j(na,1,1); bb = j(nb,1,1); bc = j(nc,1,1);
	xfe = block(ba,ba,ba,ba,ba,ba,ba,bb,bc);

	y = colvec(Za[,1:7]`)//colvec(Zb[,1]`)//colvec(Zc[,1]`);
	x = (colvec(Za[,8:14]`)//colvec(Zb[,2]`)//colvec(Zc[,2]`))||xfe;
	beta = inv(x`*x)*x`*y; e = y-x*beta;
	F[1,1] = beta[1]; F[1,2:10] = 100*beta[2:10]`; F[1,11] = 100*(1 - e`*e/(y`*(I(nrow(y))-(1/nrow(y))*j(nrow(y),1,1)*j(1,nrow(y),1))*y));

	Xa = j(na,14,0); Xb = j(nb,2,0); Xc = j(nc,2,0); 
	
	do i = 1 to iterin; 
		u1 = j(na-nb,1); call randseed(47,0); call randgen(u1, "Uniform"); k1 = ceil((na-nb)*u1);
		u2 = j(nb-nc,1); call randgen(u2, "Uniform"); k2 = ceil((nb-nc)*u2);
		u3 = j(nc,1); call randgen(u3, "Uniform"); k3 = ceil(nc*u3);

		do j = 1 to nrow(k1); Xa[j,] = Za[k1[j,],]; end;
		do j = 1 to nrow(k2); Xa[na-nb+j,] = Za[na-nb+k2[j,],]; Xb[j,] = Zb[k2[j,],]; end;
		do j = 1 to nrow(k3); Xa[na-nc+j,] = Za[na-nc+k3[j,],]; Xb[nb-nc+j,] = Zb[nb-nc+k3[j,],]; Xc[j,] = Zc[k3[j,],]; end;
 
		y = colvec(Xa[,1:7]`)//colvec(Xb[,1]`)//colvec(Xc[,1]`);
		x = (colvec(Xa[,8:14]`)//colvec(Xb[,2]`)//colvec(Xc[,2]`))||xfe;
		B[i,] = (inv(x`*x)*x`*y)`;
	end;
	se = std(B); 
	F[2,1] = se[1]; F[2,2:10] = 100*se[1,2:10]; 
	do i = 1 to nrow(B); do j = 1 to ncol(B); if B[i,j] < 0 then Btest[i,j] = 1; end; end;
	F[3,1] = Btest[:,1];
	pvdata = Btest[:,2:10]; do i = 1 to 9; pvdata[i] = min(2*pvdata[i],2*(1-pvdata[i])); end; minpvdata = min(pvdata);

	names0 = {coeff se pval}; names1 = {gamaw awp ce nce td cd pen re ad md rsq}; 
	print F[label = "Multiple asset classes, Covariance (1)" format = 6.3 rowname=names0 colname=names1];

	Za[,1:7] = Za[,1:7]-j(na,1,1)*beta[2:8]`;
	Zb[,1] = Zb[,1]-j(nb,1,1)*beta[9]`;
	Zc[,1] = Zc[,1]-j(nc,1,1)*beta[10]`;

	Xa2 = j(na,14,0); Xb2 = j(nb,2,0); Xc2 = j(nc,2,0);
	
	do i = 1 to iterout; 
		u1 = j(na-nb,1); call randgen(u1, "Uniform"); k1 = ceil((na-nb)*u1);
		u2 = j(nb-nc,1); call randgen(u2, "Uniform"); k2 = ceil((nb-nc)*u2);
		u3 = j(nc,1); call randgen(u3, "Uniform"); k3 = ceil(nc*u3);

		do j = 1 to nrow(k1); Xa[j,] = Za[k1[j,],]; end;
		do j = 1 to nrow(k2); Xa[na-nb+j,] = Za[na-nb+k2[j,],]; Xb[j,] = Zb[k2[j,],]; end;
		do j = 1 to nrow(k3); Xa[na-nc+j,] = Za[na-nc+k3[j,],]; Xb[nb-nc+j,] = Zb[nb-nc+k3[j,],]; Xc[j,] = Zc[k3[j,],]; end;
 
		y = colvec(Xa[,1:7]`)//colvec(Xb[,1]`)//colvec(Xc[,1]`);
		x = (colvec(Xa[,8:14]`)//colvec(Xb[,2]`)//colvec(Xc[,2]`))||xfe;
		beta = inv(x`*x)*x`*y;
		A[i,1:9] = beta[2:10]`;

			do k = 1 to iterin;
			u1 = j(na-nb,1); call randgen(u1, "Uniform"); k1 = ceil((na-nb)*u1);
			u2 = j(nb-nc,1); call randgen(u2, "Uniform"); k2 = ceil((nb-nc)*u2);
			u3 = j(nc,1); call randgen(u3, "Uniform"); k3 = ceil(nc*u3);

			do j = 1 to nrow(k1); Xa2[j,] = Xa[k1[j,],]; end;
			do j = 1 to nrow(k2); Xa2[na-nb+j,] = Xa[na-nb+k2[j,],]; Xb2[j,] = Xb[k2[j,],]; end;
			do j = 1 to nrow(k3); Xa2[na-nc+j,] = Xa[na-nc+k3[j,],]; Xb2[nb-nc+j,] = Xb[nb-nc+k3[j,],]; Xc2[j,] = Xc[k3[j,],]; end;
 		
			y = colvec(Xa2[,1:7]`)//colvec(Xb2[,1]`)//colvec(Xc2[,1]`);
			x = (colvec(Xa2[,8:14]`)//colvec(Xb2[,2]`)//colvec(Xc2[,2]`))||xfe;

			B[k,] = (inv(x`*x)*x`*y)`;
			end;

		Btest = j(iterin,10,0);
		do k = 1 to nrow(B); do l = 1 to ncol(B); if B[k,l] < 0 then Btest[k,l] = 1; end; end;
		A[i,10:18] = Btest[:,2:10]; do k = 10 to 18; A[i,k] = min(2*A[i,k],2*(1-A[i,k])); end; A[i,19] = min(A[i,10:18]); if A[i,19] < minpvdata then A[i,20] = 1;
	end;
	
	G = A[:,20];
	names1 = {MinPVTest}; 
	print G[label = "Extreme p-value test" format = 6.3 colname=names1];
quit;
data co.temp3; set co.temp3; if _n_ = 1 then delete; run;
data co.temp4; set co.temp4; if _n_ = 1 then delete; run;
data co.temp5; set co.temp5; if _n_ = 1 then delete; run;
proc iml;
	use co.temp3; read all var{mkt2 ce nce td cd pen re  vaw covce covnce covtd covcd covpen covre  lmkt lce lnce ltd lcd lpen lre} into Za;
	use co.temp4; read all var{ad covad lad} into Zb;
	use co.temp5; read all var{md covmd lmd} into Zc;

	F = j(3,11,{.});
	iterout = 10000;
	iterin = 10000;

	A = j(iterout,20,0);
	B = j(iterin,10,0); Btest = j(iterin,10,0);

	na = nrow(Za); nb = nrow(Zb); nc = nrow(Zc);
	ba = j(na,1,1); bb = j(nb,1,1); bc = j(nc,1,1);
	xfe = block(ba,ba,ba,ba,ba,ba,ba,bb,bc);

	y = colvec(Za[,1:7]`)//colvec(Zb[,1]`)//colvec(Zc[,1]`);
	x = (colvec(Za[,8:14]`)//colvec(Zb[,2]`)//colvec(Zc[,2]`))||xfe||block(Za[,15],Za[,16],Za[,17],Za[,18],Za[,19],Za[,20],Za[,21],Zb[,3],Zc[,3]);
	beta = inv(x`*x)*x`*y; e = y-x*beta;
	F[1,1] = beta[1]; F[1,2:10] = 100*beta[2:10]`; F[1,11] = 100*(1 - e`*e/(y`*(I(nrow(y))-(1/nrow(y))*j(nrow(y),1,1)*j(1,nrow(y),1))*y));

	Xa = j(na,21,0); Xb = j(nb,3,0); Xc = j(nc,3,0); 
	
	do i = 1 to iterin; 
		u1 = j(na-nb,1); call randseed(47,0); call randgen(u1, "Uniform"); k1 = ceil((na-nb)*u1);
		u2 = j(nb-nc,1); call randgen(u2, "Uniform"); k2 = ceil((nb-nc)*u2);
		u3 = j(nc,1); call randgen(u3, "Uniform"); k3 = ceil(nc*u3);

		do j = 1 to nrow(k1); Xa[j,] = Za[k1[j,],]; end;
		do j = 1 to nrow(k2); Xa[na-nb+j,] = Za[na-nb+k2[j,],]; Xb[j,] = Zb[k2[j,],]; end;
		do j = 1 to nrow(k3); Xa[na-nc+j,] = Za[na-nc+k3[j,],]; Xb[nb-nc+j,] = Zb[nb-nc+k3[j,],]; Xc[j,] = Zc[k3[j,],]; end;
 
		y = colvec(Xa[,1:7]`)//colvec(Xb[,1]`)//colvec(Xc[,1]`);
		x = (colvec(Xa[,8:14]`)//colvec(Xb[,2]`)//colvec(Xc[,2]`))||xfe||block(Xa[,15],Xa[,16],Xa[,17],Xa[,18],Xa[,19],Xa[,20],Xa[,21],Xb[,3],Xc[,3]);
		bt = inv(x`*x)*x`*y; B[i,] = bt[1:10,1]`;
	end;
	se = std(B); 
	F[2,1] = se[1]; F[2,2:10] = 100*se[1,2:10]; 
	do i = 1 to nrow(B); do j = 1 to ncol(B); if B[i,j] < 0 then Btest[i,j] = 1; end; end;
	F[3,1] = Btest[:,1];
	pvdata = Btest[:,2:10]; do i = 1 to 9; pvdata[i] = min(2*pvdata[i],2*(1-pvdata[i])); end; minpvdata = min(pvdata);

	names0 = {coeff se pval}; names1 = {gamaw awp ce nce td cd pen re ad md rsq}; 
	print F[label = "Multiple asset classes, Covariance (1)" format = 6.3 rowname=names0 colname=names1];

	Za[,1:7] = Za[,1:7]-j(na,1,1)*beta[2:8]`; Za[,15:21] = Za[,15:21]-j(na,1,1)*beta[2:8]`;
	Zb[,1] = Zb[,1]-j(nb,1,1)*beta[9]`; Zb[,3] = Zb[,3]-j(nb,1,1)*beta[9]`;
	Zc[,1] = Zc[,1]-j(nc,1,1)*beta[10]`; Zc[,3] = Zc[,3]-j(nc,1,1)*beta[10]`;

	Xa2 = j(na,21,0); Xb2 = j(nb,3,0); Xc2 = j(nc,3,0);
	
	do i = 1 to iterout; 
		u1 = j(na-nb,1); call randgen(u1, "Uniform"); k1 = ceil((na-nb)*u1);
		u2 = j(nb-nc,1); call randgen(u2, "Uniform"); k2 = ceil((nb-nc)*u2);
		u3 = j(nc,1); call randgen(u3, "Uniform"); k3 = ceil(nc*u3);

		do j = 1 to nrow(k1); Xa[j,] = Za[k1[j,],]; end;
		do j = 1 to nrow(k2); Xa[na-nb+j,] = Za[na-nb+k2[j,],]; Xb[j,] = Zb[k2[j,],]; end;
		do j = 1 to nrow(k3); Xa[na-nc+j,] = Za[na-nc+k3[j,],]; Xb[nb-nc+j,] = Zb[nb-nc+k3[j,],]; Xc[j,] = Zc[k3[j,],]; end;
 
		y = colvec(Xa[,1:7]`)//colvec(Xb[,1]`)//colvec(Xc[,1]`);
		x = (colvec(Xa[,8:14]`)//colvec(Xb[,2]`)//colvec(Xc[,2]`))||xfe||block(Xa[,15],Xa[,16],Xa[,17],Xa[,18],Xa[,19],Xa[,20],Xa[,21],Xb[,3],Xc[,3]);
		bt = inv(x`*x)*x`*y; A[i,1:9] = bt[2:10,1]`;

			do k = 1 to iterin;
			u1 = j(na-nb,1); call randgen(u1, "Uniform"); k1 = ceil((na-nb)*u1);
			u2 = j(nb-nc,1); call randgen(u2, "Uniform"); k2 = ceil((nb-nc)*u2);
			u3 = j(nc,1); call randgen(u3, "Uniform"); k3 = ceil(nc*u3);

			do j = 1 to nrow(k1); Xa2[j,] = Xa[k1[j,],]; end;
			do j = 1 to nrow(k2); Xa2[na-nb+j,] = Xa[na-nb+k2[j,],]; Xb2[j,] = Xb[k2[j,],]; end;
			do j = 1 to nrow(k3); Xa2[na-nc+j,] = Xa[na-nc+k3[j,],]; Xb2[nb-nc+j,] = Xb[nb-nc+k3[j,],]; Xc2[j,] = Xc[k3[j,],]; end;
 		
			y = colvec(Xa2[,1:7]`)//colvec(Xb2[,1]`)//colvec(Xc2[,1]`);
			x = (colvec(Xa2[,8:14]`)//colvec(Xb2[,2]`)//colvec(Xc2[,2]`))||xfe||block(Xa2[,15],Xa2[,16],Xa2[,17],Xa2[,18],Xa2[,19],Xa2[,20],Xa2[,21],Xb2[,3],Xc2[,3]);
			bt = inv(x`*x)*x`*y; B[k,] = bt[1:10,1]`;
			end;

		Btest = j(iterin,10,0);
		do k = 1 to nrow(B); do l = 1 to ncol(B); if B[k,l] < 0 then Btest[k,l] = 1; end; end;
		A[i,10:18] = Btest[:,2:10]; do k = 10 to 18; A[i,k] = min(2*A[i,k],2*(1-A[i,k])); end; A[i,19] = min(A[i,10:18]); if A[i,19] < minpvdata then A[i,20] = 1;
	end;
	
	G = A[:,20];
	names1 = {MinPVTest}; 
	print G[label = "Extreme p-value test" format = 6.3 colname=names1];
quit;
proc datasets noprint lib = co; delete temp1-temp5; quit;


* Tab. 05: The risk-return tradeoff across anomaly portfolios: Covariance approach;
* NOTE: Need to adjust code to specify the desired test assets (1 = size, 2 = b/m, 3 = momentum) and market proxy (1 = aggregate wealth port., 2 = stock market portfolio);

data co.temp1; set co.portfolios; s1 = 0.01*s1; s2 = 0.01*s2; s3 = 0.01*s3; s4 = 0.01*s4; s5 = 0.01*s5; s6 = 0.01*s6; s7 = 0.01*s7; s8 = 0.01*s8; s9 = 0.01*s9; s10 = 0.01*s10; run;
data co.temp1; set co.temp1; ls1 = lag(s1); ls2 = lag(s2); ls3 = lag(s3); ls4 = lag(s4); ls5 = lag(s5); ls6 = lag(s6); ls7 = lag(s7); ls8 = lag(s8); ls9 = lag(s9); ls10 = lag(s10); run;
data co.temp2; set co.portfolios; b1 = 0.01*b1; b2 = 0.01*b2; b3 = 0.01*b3; b4 = 0.01*b4; b5 = 0.01*b5; b6 = 0.01*b6; b7 = 0.01*b7; b8 = 0.01*b8; b9 = 0.01*b9; b10 = 0.01*b10; run;
data co.temp2; set co.temp2; lb1 = lag(b1); lb2 = lag(b2); lb3 = lag(b3); lb4 = lag(b4); lb5 = lag(b5); lb6 = lag(b6); lb7 = lag(b7); lb8 = lag(b8); lb9 = lag(b9); lb10 = lag(b10); run;
data co.temp3; set co.portfolios; m1 = 0.01*m1; m2 = 0.01*m2; m3 = 0.01*m3; m4 = 0.01*m4; m5 = 0.01*m5; m6 = 0.01*m6; m7 = 0.01*m7; m8 = 0.01*m8; m9 = 0.01*m9; m10 = 0.01*m10; run;
data co.temp3; set co.temp3; lm1 = lag(m1); lm2 = lag(m2); lm3 = lag(m3); lm4 = lag(m4); lm5 = lag(m5); lm6 = lag(m6); lm7 = lag(m7); lm8 = lag(m8); lm9 = lag(m9); lm10 = lag(m10); run; 
data co.temp4; set co.bekk_anomalies_size_awpproxy; run;
data co.temp5; set co.bekk_anomalies_size_stocksproxy; run;
data co.temp6; set co.bekk_anomalies_bm_awpproxy; run;
data co.temp7; set co.bekk_anomalies_bm_stocksproxy; run;
data co.temp8; set co.bekk_anomalies_mom_awpproxy; run;
data co.temp9; set co.bekk_anomalies_mom_stocksproxy; run;
proc iml;
	use co.temp1; read all var{s10 s9 s8 s7 s6 s5 s4 s3 s2 s1} into Z1;  use co.temp4; read all var{cov10 cov9 cov8 cov7 cov6 cov5 cov4 cov3 cov2 cov1} into A1;  use co.temp5; read all var{cov10 cov9 cov8 cov7 cov6 cov5 cov4 cov3 cov2 cov1} into S1;
	use co.temp2; read all var{b1 b2 b3 b4 b5 b6 b7 b8 b9 b10} into Z2;  use co.temp6; read all var{cov1 cov2 cov3 cov4 cov5 cov6 cov7 cov8 cov9 cov10} into A2;  use co.temp7; read all var{cov1 cov2 cov3 cov4 cov5 cov6 cov7 cov8 cov9 cov10} into S2;
	use co.temp3; read all var{m1 m2 m3 m4 m5 m6 m7 m8 m9 m10} into Z3;  use co.temp8; read all var{cov1 cov2 cov3 cov4 cov5 cov6 cov7 cov8 cov9 cov10} into A3;  use co.temp9; read all var{cov1 cov2 cov3 cov4 cov5 cov6 cov7 cov8 cov9 cov10} into S3;

	ports = 1; /* Choose portfolios:   (1) size, (2) b/m, (3) mom */
	proxy = 1; /* Choose market proxy: (1) awp,  (2) stocks       */

	if ports = 1 then do; 
		if proxy = 1 then do; Z = Z1[,1:10]||A1[,1:10]; end;
		if proxy = 2 then do; Z = Z1[,1:10]||S1[,1:10]; end; end;
	if ports = 2 then do; 
		if proxy = 1 then do; Z = Z2[,1:10]||A2[,1:10]; end;
		if proxy = 2 then do; Z = Z2[,1:10]||S2[,1:10]; end; end;
	if ports = 3 then do; 
		if proxy = 1 then do; Z = Z3[,1:10]||A3[,1:10]; end;
		if proxy = 2 then do; Z = Z3[,1:10]||S3[,1:10]; end; end;

	F = j(3,13,{.});
	iterout = 10000;
	iterin = 10000;

	A = j(iterout,22,0);
	B = j(iterin,12,0); Btest = j(iterin,12,0);

	n = nrow(Z); 
	do i = 1 to 10; if i = 1 then xfe = j(n,1,1); else xfe = block(xfe,j(n,1,1)); end;

	y = colvec(Z[,1:10]`);
	x = colvec(Z[,11:20]`)||xfe;
	beta = inv(x`*x)*x`*y; e = y-x*beta;
	F[1,1] = beta[1]; F[1,2:11] = 100*beta[2:11]`; F[1,12] = F[1,11]-F[1,2]; F[1,13] = 100*(1 - e`*e/(y`*(I(nrow(y))-(1/nrow(y))*j(nrow(y),1,1)*j(1,nrow(y),1))*y));

	Xa = j(n,20,0);  
	
	do i = 1 to iterin; 
		u = j(n,1); call randseed(47,0); call randgen(u, "Uniform"); k = ceil(n*u);
		do j = 1 to nrow(k); Xa[j,] = Z[k[j,],]; end;
 
		y = colvec(Xa[,1:10]`);
		x = colvec(Xa[,11:20]`)||xfe;
		B[i,1:11] = (inv(x`*x)*x`*y)`; B[i,12] = B[i,11]-B[i,2];
	end;
	se = std(B); 
	F[2,1] = se[1]; F[2,2:12] = 100*se[1,2:12]; 
	do i = 1 to nrow(B); do j = 1 to ncol(B); if B[i,j] < 0 then Btest[i,j] = 1; end; end;
	F[3,1] = Btest[:,1]; F[3,12] = Btest[:,12];
	pvdata = Btest[:,2:11]; do i = 1 to 10; pvdata[i] = min(2*pvdata[i],2*(1-pvdata[i])); end; minpvdata = min(pvdata);

	if ports = 1 then do; names1 = {gam s10 s9 s8 s7 s6 s5 s4 s3 s2 s1 diff rsq}; end;
	if ports = 2 then do; names1 = {gam b1 b2 b3 b4 b5 b6 b7 b8 b9 b10 diff rsq}; end;
	if ports = 3 then do; names1 = {gam m1 m2 m3 m4 m5 m6 m7 m8 m9 m10 diff rsq}; end;

	names0 = {awp se pval stocks se pval};  
	print F[label = "Size BM Mom portfolios (No lag)" format = 6.3 rowname=names0 colname=names1];

	Z[,1:10] = Z[,1:10]-j(n,1,1)*beta[2:11]`;
	Xa2 = j(n,20,0); 
	B = j(iterin,11,0);

	do i = 1 to iterout; 
		u = j(n,1); call randgen(u, "Uniform"); k = ceil(n*u);
		do j = 1 to nrow(k); Xa[j,] = Z[k[j,],]; end;
		
		y = colvec(Xa[,1:10]`);
		x = colvec(Xa[,11:20]`)||xfe;
		beta = inv(x`*x)*x`*y;
		A[i,1:10] = beta[2:11]`;

			do k = 1 to iterin;
			u1 = j(n,1); call randgen(u, "Uniform"); k1 = ceil(n*u);
			do j = 1 to nrow(k1); Xa2[j,] = Xa[k1[j,],]; end;
 		
			y = colvec(Xa2[,1:10]`);
			x = colvec(Xa2[,11:20]`)||xfe;

			B[k,] = (inv(x`*x)*x`*y)`;
			end;

		Btest = j(iterin,11,0);
		do k = 1 to nrow(B); do l = 1 to ncol(B); if B[k,l] < 0 then Btest[k,l] = 1; end; end;
		A[i,11:20] = Btest[:,2:11]; do k = 11 to 20; A[i,k] = min(2*A[i,k],2*(1-A[i,k])); end; A[i,21] = min(A[i,11:20]); if A[i,21] < minpvdata then A[i,22] = 1;
	end;
	
	G = A[:,22];
	names1 = {MinPVTest}; 
	print G[label = "Extreme p-value test" format = 6.3 colname=names1];
quit;
data co.temp1; set co.temp1; if _n_ = 1 then delete; run;
data co.temp2; set co.temp2; if _n_ = 1 then delete; run;
data co.temp3; set co.temp3; if _n_ = 1 then delete; run;
data co.temp4; set co.temp4; if _n_ = 1 then delete; run;
data co.temp5; set co.temp5; if _n_ = 1 then delete; run;
data co.temp6; set co.temp6; if _n_ = 1 then delete; run;
data co.temp7; set co.temp7; if _n_ = 1 then delete; run;
data co.temp8; set co.temp8; if _n_ = 1 then delete; run;
data co.temp9; set co.temp9; if _n_ = 1 then delete; run;
proc iml;
	use co.temp1; read all var{s10 s9 s8 s7 s6 s5 s4 s3 s2 s1} into Z1;  use co.temp1; read all var{ls10 ls9 ls8 ls7 ls6 ls5 ls4 ls3 ls2 ls1} into L1;  use co.temp4; read all var{cov10 cov9 cov8 cov7 cov6 cov5 cov4 cov3 cov2 cov1} into A1;  use co.temp5; read all var{cov10 cov9 cov8 cov7 cov6 cov5 cov4 cov3 cov2 cov1} into S1;
	use co.temp2; read all var{b1 b2 b3 b4 b5 b6 b7 b8 b9 b10} into Z2;  use co.temp2; read all var{lb1 lb2 lb3 lb4 lb5 lb6 lb7 lb8 lb9 lb10} into L2;  use co.temp6; read all var{cov1 cov2 cov3 cov4 cov5 cov6 cov7 cov8 cov9 cov10} into A2;  use co.temp7; read all var{cov1 cov2 cov3 cov4 cov5 cov6 cov7 cov8 cov9 cov10} into S2;
	use co.temp3; read all var{m1 m2 m3 m4 m5 m6 m7 m8 m9 m10} into Z3;  use co.temp3; read all var{lm1 lm2 lm3 lm4 lm5 lm6 lm7 lm8 lm9 lm10} into L3;  use co.temp8; read all var{cov1 cov2 cov3 cov4 cov5 cov6 cov7 cov8 cov9 cov10} into A3;  use co.temp9; read all var{cov1 cov2 cov3 cov4 cov5 cov6 cov7 cov8 cov9 cov10} into S3;

	ports = 1; /* Choose portfolios:   (1) size, (2) b/m, (3) mom */
	proxy = 1; /* Choose market proxy: (1) awp,  (2) stocks       */

	if ports = 1 then do; 
		if proxy = 1 then do; Z = Z1[,1:10]||A1[,1:10]||L1[,1:10]; end;
		if proxy = 2 then do; Z = Z1[,1:10]||S1[,1:10]||L1[,1:10]; end; end;
	if ports = 2 then do; 
		if proxy = 1 then do; Z = Z2[,1:10]||A2[,1:10]||L2[,1:10]; end;
		if proxy = 2 then do; Z = Z2[,1:10]||S2[,1:10]||L2[,1:10]; end; end;
	if ports = 3 then do; 
		if proxy = 1 then do; Z = Z3[,1:10]||A3[,1:10]||L3[,1:10]; end;
		if proxy = 2 then do; Z = Z3[,1:10]||S3[,1:10]||L3[,1:10]; end; end;

	F = j(3,13,{.});
	iterout = 10000;
	iterin = 10000;

	A = j(iterout,22,0);
	B = j(iterin,12,0); Btest = j(iterin,12,0);

	n = nrow(Z); 
	do i = 1 to 10; if i = 1 then xfe = j(n,1,1); else xfe = block(xfe,j(n,1,1)); end;

	y = colvec(Z[,1:10]`);
	do p = 1 to 10; if p = 1 then xlag = Z[,21]; else xlag = block(xlag,Z[,20+p]); end;
	x = colvec(Z[,11:20]`)||xfe||xlag;
	beta = inv(x`*x)*x`*y; e = y-x*beta;
	F[1,1] = beta[1]; F[1,2:11] = 100*beta[2:11]`; F[1,12] = F[1,11]-F[1,2]; F[1,13] = 100*(1 - e`*e/(y`*(I(nrow(y))-(1/nrow(y))*j(nrow(y),1,1)*j(1,nrow(y),1))*y));

	Xa = j(n,30,0);  
	
	do i = 1 to iterin; 
		u = j(n,1); call randseed(47,0); call randgen(u, "Uniform"); k = ceil(n*u);
		do j = 1 to nrow(k); Xa[j,] = Z[k[j,],]; end;
 
		y = colvec(Xa[,1:10]`);
		do p = 1 to 10; if p = 1 then xlag = Xa[,21]; else xlag = block(xlag,Xa[,20+p]); end;
		x = colvec(Xa[,11:20]`)||xfe||xlag;
		bt = inv(x`*x)*x`*y; B[i,1:11] = bt[1:11,1]`; B[i,12] = B[i,11]-B[i,2];
	end;
	se = std(B); 
	F[2,1] = se[1]; F[2,2:12] = 100*se[1,2:12]; 
	do i = 1 to nrow(B); do j = 1 to ncol(B); if B[i,j] < 0 then Btest[i,j] = 1; end; end;
	F[3,1] = Btest[:,1]; F[3,12] = Btest[:,12];
	pvdata = Btest[:,2:11]; do i = 1 to 10; pvdata[i] = min(2*pvdata[i],2*(1-pvdata[i])); end; minpvdata = min(pvdata);

	if ports = 1 then do; names1 = {gam s10 s9 s8 s7 s6 s5 s4 s3 s2 s1 diff rsq}; end;
	if ports = 2 then do; names1 = {gam b1 b2 b3 b4 b5 b6 b7 b8 b9 b10 diff rsq}; end;
	if ports = 3 then do; names1 = {gam m1 m2 m3 m4 m5 m6 m7 m8 m9 m10 diff rsq}; end;

	names0 = {awp se pval stocks se pval};  
	print F[label = "Size BM Mom portfolios (Lag)" format = 6.3 rowname=names0 colname=names1];


	Z[,1:10] = Z[,1:10]-j(n,1,1)*beta[2:11]`; Z[,21:30] = Z[,21:30]-j(n,1,1)*beta[2:11]`;
	Xa2 = j(n,30,0); 
	B = j(iterin,11,0);

	do i = 1 to iterout; 
		u = j(n,1); call randgen(u, "Uniform"); k = ceil(n*u);
		do j = 1 to nrow(k); Xa[j,] = Z[k[j,],]; end;
		
		y = colvec(Xa[,1:10]`);
		do p = 1 to 10; if p = 1 then xlag = Xa[,21]; else xlag = block(xlag,Xa[,20+p]); end;
		x = colvec(Xa[,11:20]`)||xfe||xlag;
		beta = inv(x`*x)*x`*y;
		A[i,1:10] = beta[2:11]`;

			do k = 1 to iterin;
			u1 = j(n,1); call randgen(u, "Uniform"); k1 = ceil(n*u);
			do j = 1 to nrow(k1); Xa2[j,] = Xa[k1[j,],]; end;
 		
			y = colvec(Xa2[,1:10]`);
			do p = 1 to 10; if p = 1 then xlag = Xa2[,21]; else xlag = block(xlag,Xa2[,20+p]); end;
			x = colvec(Xa2[,11:20]`)||xfe||xlag;
			bt = inv(x`*x)*x`*y; B[k,] = bt[1:11,1]`;
			end;

		Btest = j(iterin,11,0);
		do k = 1 to nrow(B); do l = 1 to ncol(B); if B[k,l] < 0 then Btest[k,l] = 1; end; end;
		A[i,11:20] = Btest[:,2:11]; do k = 11 to 20; A[i,k] = min(2*A[i,k],2*(1-A[i,k])); end; A[i,21] = min(A[i,11:20]); if A[i,21] < minpvdata then A[i,22] = 1;
	end;
	
	G = A[:,22];
	names1 = {MinPVTest}; 
	print G[label = "Extreme p-value test" format = 6.3 colname=names1];
quit;
proc datasets noprint lib = co; delete temp1-temp9; quit;
